/*
** Parse_Filter_Conf.c for Gmurf in ~/Gmurf/Charge_Filter/
** 
** Made by 
** E-mail   <ahes@ker-ys.epita.fr>
** 
** Started on  Mon Mar  8 22:51:54 1999 
** Last update Sat Mar 13 03:28:27 1999 
*/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "Errors.h"
#include "Messages.h"
#include "Constants.h"
#include "Parse_Filter_Conf.h"

/*
** is_nb: returne 1 si le caractere est un chiffre, 0 sinon
*/
int	is_nb(data)
char	data;
{
  if ((data >= '0') && (data <= '9'))
    return (1);
  else
    return (0);
}

/*
** print_error: affiche un message d'erreur lie a l'ecriture du fichier a parser et quite Gmurf
*/
void	print_error(error, line, buf)
char	*error;
int	line;
char	*buf;
{
  printf("%s, %s:\n   %s\n %s %d: %s %s,\n%s\n", GMURF_ERROR, BAD_SCRIPT, FILTER_LOADER, BAD_LINE, line, buf, BAD_FILTER_DECLARATION, error);
  exit(-1);
}

/*
** is_blank_line: retourne 1 si la ligne est une ligne vide, 0 sinon
*/
int	is_blank_line(buf)
char	*buf;
{
  int	cpt;
  
  cpt = 0;
  while ((buf[cpt] == ' ') && (cpt < MAX_LINE_LENGTH))
       cpt++;
  if ((buf[cpt] == '\n') || (buf[cpt] == '\0') || (buf[cpt] == '#') || (cpt == MAX_LINE_LENGTH))
    return (1);
  else
      return (0);
}

/*
** get_str: extrait une chaine de caractere a partir de la position donnee jusqu'a la fin donnee
*/
char	*get_str(buf, begin, end)
char	*buf;
int	begin;
int	end;
{
  int	cpt;
  char	*result;
  
  if (begin > end)
    return (NULL);
  result = (char *) malloc (end - begin + 2);	/* ici, jadis il y eu en +1 a la place d'un +2 qui m'a */ 
  for (cpt = begin; cpt <= end; cpt++)		/* fait chier pendant 2 jours */
    result[cpt - begin] = buf[cpt];
  result[cpt - begin] = '\0';
  return (result);
}

/*
** create_new_filter_loader: cree un nouveau fichier (vide) contenant les chemin des plug-ins
**			     relatifs aux filtres sonores
*/
FILE	*create_new_filter_loader()
{
  FILE	*temp_file;

  printf("%s %s\n", FILE_NON_EXISTENT, FILTER_LOADER);
  printf("...%s\n", FILE_CREAT);
  if ((temp_file = fopen(FILTER_LOADER, "a+")) == NULL)
    printf("%s\n", BAD_CREAT);
  return (temp_file);
}

/*
** get_category_declaration: regarde si la ligne en question correspond au debut de la declaration
##			     des categories de filtres sonores
*/
int	get_category_declaration(buf, line)
char	*buf;
int	line;
{
  int	cpt;
  char	*str;
  
  if (buf[0] != '$')
    print_error(NO_START_CATEGORY_DECLARATION, line, buf);
  cpt = 0;
  while ((buf[cpt] != ' ') && (buf[cpt != '\n']) && (buf[cpt] != '\0') && (cpt < MAX_LINE_LENGTH))
    cpt++;
  str = (char *) malloc(cpt + 1);
  str[cpt] = '\0';
  while (cpt >= 0)
    str[cpt] = buf[cpt--];
  if (strncmp(START_CATEGORY, str, strlen(START_CATEGORY)) != 0)
    print_error(NO_START_CATEGORY_DECLARATION, line, buf);
  return (1);
}

/*
** add_new_category: ajoute une nouvelle categorie dans la liste
*/
void		add_new_category(filters, category_name, category_nb)
t_filters	*filters;
char		*category_name;
int		category_nb;
{
  t_category	*category;

  filters->nb_category += 1;
  category		= (t_category *) malloc(sizeof(t_category));
  category->nb_filters	= 0;
  category->name	= category_name;
  category->filter	= NULL;
  category->next	= filters->category;
  filters->category	= category;
}

/*
** get_category: lit une categorie de type de filtres
*/
int		get_category(buf, line, filters)
char		*buf;
int		line;
t_filters	*filters;
{
  int		cpt;
  int		begin;
  int		category_nb;
  char		*category_nb_str;
  char		*category_name;
  static int	max_category_nb = 0;

  if (strncmp(END_CATEGORY, buf, strlen(END_CATEGORY)) == 0)
    return (1);
  if (buf[0] != '$') /* si pas '$' en premier caractere, alors quite */
    print_error(BAD_CATEGORY_START, line, buf);
  if (buf[1] != '"')
    print_error(NO_CATEGORY_START_NAME, line, buf);
  cpt = 2;
  while ((buf[cpt] != '"') && (cpt < MAX_LINE_LENGTH))
    cpt++;
  if ((cpt >= MAX_LINE_LENGTH) || (cpt == 2))
    print_error((cpt == 2? CATEGORY_NAME_EMPTY:NO_CLOSE_CHAR), line, buf);
  category_name = get_str(buf, 2, cpt - 1);
  while ((cpt < MAX_LINE_LENGTH) && !((is_nb(buf[cpt]) == 1) && (buf[cpt - 1] == ' ')))
    cpt++;
  if (cpt >= MAX_LINE_LENGTH)
    print_error(NO_CATEGORY_NB, line, buf);
  begin = cpt;
  while ((cpt < MAX_LINE_LENGTH) && (is_nb(buf[cpt]) == 1))
    cpt++;
  if (cpt >= MAX_LINE_LENGTH)
    print_error(BAD_CATEGORY_NB, line, buf);
  category_nb_str = get_str(buf, begin, cpt - 1);
  category_nb = atoi(category_nb_str);
  if (category_nb - 1 != max_category_nb++)
    print_error(WRONG_CATEGORY_NB, line, buf);
  free(category_nb_str);
  add_new_category(filters, category_name, category_nb);
  return (0);
}

/*
** add_new_filter: ajoute un filtre dans la liste chaine
*/

void		add_new_filter(filters, category_nb, filter_name, filter_path)
t_filters	*filters;
int		category_nb;
char		*filter_name;
char		*filter_path;
{
  int		cpt;
  t_filter	*filter;
  t_category	*category;

  cpt = filters->nb_category;
  category = filters->category;
  while (cpt > category_nb)
    {
      category = category->next;
      cpt--;
    }
  (category->nb_filters)++;
  filter = (t_filter *) malloc(sizeof(t_filter));
  filter->name = filter_name;
  filter->path = filter_path;
  filter->next = category->filter;
#ifdef TEXT_MSG
  printf("ajoute %s a %s\n", filter->name, category->name);
#endif
  category->filter = filter;
}

/*
** get_nb_category: renvoie le numero de categorie du filtre
*/
int	get_nb_category(buf, line, cpt)
char*	buf;
int	line;
int	*cpt;
{
  int	category_nb;
  char	*category_nb_str;
  
  if (is_nb(buf[0]) != 1)
    print_error(NO_CATEGORY_FOR_FILTER, line, buf);
  while ((is_nb(buf[*cpt] == 1)) && (*cpt < MAX_LINE_LENGTH))
    (*cpt)++; 
  if ((*cpt >= MAX_LINE_LENGTH) || (buf[*cpt] != ' '))
    print_error(BAD_CATEGORY_NB_FOR_FILTER, line, buf); 
  category_nb_str = get_str(buf, 0, *cpt - 1); 
  category_nb = atoi(category_nb_str); 
  free(category_nb_str); 

  return (category_nb);
}

/*
** add_filter2liste: ajoute un filtre dans la liste des filtres
*/
void		add_filter2liste(filters, buf, line)
t_filters	*filters;
char		*buf;
int		line;
{
  int	cpt;
  int	begin;
  int	category_nb;
  char	*filter_path;
  char	*filter_name;

  cpt = 1;
  category_nb = get_nb_category(buf, line, &cpt);
  if ((category_nb <= 0) || (category_nb > filters->nb_category))
    print_error(BAD_VALUE_FOR_CATEGORY_NB, line, buf);
  while ((buf[cpt] != '"') && (cpt < MAX_LINE_LENGTH))
    cpt++; 
  if (cpt >= MAX_LINE_LENGTH)
    print_error(NO_FILTER_NAME, line, buf); 
  begin = ++cpt;
  while ((buf[cpt] != '"') && (cpt < MAX_LINE_LENGTH))
    cpt++;
  if (cpt >= MAX_LINE_LENGTH)
    print_error(NO_CLOSE_CHAR, line, buf); 
  filter_name = get_str(buf, begin, cpt - 1);
  cpt++;
  while ((buf[cpt] != '"') && (cpt < MAX_LINE_LENGTH))
    cpt++;
  if (cpt >= MAX_LINE_LENGTH)
    print_error(NO_FILTER_PATH, line, buf);
  begin = cpt++;
  while ((buf[cpt] != '"') && (cpt < MAX_LINE_LENGTH))
    cpt++;
  if (cpt >= MAX_LINE_LENGTH)
    print_error(BAD_FILTER_PATH, line, buf);
  filter_path = get_str(buf, begin + 1, cpt - 1);
  if (filter_path == NULL)
    print_error(BAD_FILTER_PATH, line, buf);
  add_new_filter(filters, category_nb, filter_name, filter_path);
}

/*
** end_of_filter: return 1 if buf is "$filter_end" and 0 else
*/
int	end_of_filter(buf)
char	*buf;
{
  if (strncmp(buf, "$end_filter", 11) == 0)
    return (1);
  else
    return (0);
}

/*
** get_filter_path: obtient les chemins des plug-ins des filtres sonores
**		    en cas de reponse negative, retourne NULL
*/
t_filters	*get_filter_path(file)
char		*file;
{
  int		category_declaration = 0;
  int		category_end = 0;
  int		filter_end = 0;
  int		line;
  FILE		*file2test;
  char		*buf;
  t_filters	*filters;

  buf = (char *) malloc (MAX_LINE_LENGTH * sizeof(char));
  filters = (t_filters *) malloc(sizeof(t_filters));
  filters->nb_category = 0;
  filters->category    = NULL;
  if (access(file, F_OK) == -1)
    {
      file2test = create_new_filter_loader();
      return (filters);
    }
  else
    {
      file2test = fopen(file, "r");
      line = 0;
      while ((!feof(file2test)) && (filter_end == 0))
	{
	  fgets(buf, MAX_LINE_LENGTH, file2test);
	  line++;
	  if ((!is_blank_line(buf)) && (strlen(buf) >= 2))
	    {
	      if (end_of_filter(buf) == 1)
		filter_end = 1;
	      else
		{
		  if (category_end == 1)
		    add_filter2liste(filters, buf, line);
		  if ((category_declaration) && (!category_end))
		    category_end = get_category(buf, line, filters);
		  if (!category_declaration)
		    category_declaration = get_category_declaration(buf, line);
		}
	    }
	}
      if (filter_end == 0)
	print_error(NO_FILTER_END, line, buf);
      if ((filter_end == 1) && (category_end == 0))
	print_error(UNEXPECTED_END_FILTER, line, buf);
      fclose(file2test);
    }
  if (category_declaration == 0)
    print_error(NO_START_CATEGORY_DECLARATION, line, buf);
  else
    if (category_end == 0)
      print_error(NO_CATEGORY_END, line, buf);
    else
      return (filters);
  return (NULL);
}
